@@ -142,7 +142,7 @@ def get_zmatrix(atoms,zmat_id,zmat_index):
            zmatrix.append([r,ang,tor])
     return zmatrix
 
-
+ 
 def check_zmat(atoms=None,rmin=0.8,rmax=1.2,angmax=15.0,
                zmat_id=None,zmat_index=None,InitZmat=None):
     zmatrix = get_zmatrix(atoms,zmat_id,zmat_index)
@@ -299,10 +299,10 @@ class AtomDance(object):
       self.zmat_index = []
       self.zmat_id    = []
       specs           = atoms.get_chemical_symbols()
-      mols = molecules(self.natom,specs,atoms.positions,
-                       cell=atoms.cell,
-                       table=self.neighbors)
-      for m in mols:
+      self.mols       = molecules(self.natom,specs,atoms.positions,
+                                 cell=atoms.cell,
+                                 table=self.neighbors)
+      for m in self.mols:
           if len(m.mol_index)==1:
              continue
           for i in m.mol_index:
@@ -481,6 +481,7 @@ class AtomDance(object):
       atoms = self.set_bond_momenta(i,j,atoms,sign=sign)
       return atoms
       
+
   def bond_momenta(self,atoms):
       ratio = []
       self.ir.calculate_Delta(atoms)
@@ -497,7 +498,39 @@ class AtomDance(object):
              return atoms,bd,groupi,groupj
       return atoms,None,[],[]
 
-  def set_bond_momenta(self,i,j,atoms,sign=1.0):
+
+  def check_momenta(self,atoms,freeatoms=None):
+      v = atoms.get_velocities()
+
+      try:
+         f = atoms.get_forces()
+         haveforce = True
+      except:
+         haveforce = False
+      if haveforce:
+         for i in freeatoms:
+             for j in freeatoms:
+                 if (i,j) in self.InitBonds:
+                    vij = atoms.positions[j] - atoms.positions[i]
+                    rij = np.sqrt(np.sum(np.square(vij)))
+                    vij = vij/rij
+ 
+                    vi   = np.dot(v[i],vij)
+                    vj   = np.dot(v[j],vij)
+ 
+                    fi   = np.dot(f[i],vij)
+                    fj   = np.dot(f[j],vij)
+
+                    if abs(vi)<0.003 and abs(fi)<0.5:
+                       if vi>=0.0:
+                          v[i] = v[i] + 0.05*vij
+                       else:
+                          v[i] = v[i] - 0.05*vij
+      atoms.set_velocities(v)
+      return atoms
+
+
+  def set_bond_momenta(self,i,j,atoms,sign=1.0,add=False):
       ha      = int(0.5*self.natom)
       # x     = atoms.get_positions()
       v       = np.zeros([self.natom,3])
@@ -526,13 +559,21 @@ class AtomDance(object):
       vj  = 1.0/massj
 
       for a in group_i:
-          v[a] = sign*vi*vij
+          if add:
+             v[a] = v[a] + sign*vi*vij
+          else:
+             v[a] = sign*vi*vij
 
       for a in group_j:
-          v[a] = -sign*vj*vij
+          if add:
+             v[a] = v[a] - sign*vj*vij
+          else:
+             v[a] = -sign*vj*vij
+
       atoms.set_velocities(v)
       return atoms,group_i,group_j
       
+
   def zmat_relax(self,atoms,nbin=10,relax_step=None,
                  zmat_variable=None,zvlo=None,zvhi=None,
                  traj='zmat.traj'):
@@ -593,6 +634,7 @@ class AtomDance(object):
       his.close()
       return images,relaxlog
 
+
   def get_optimal_zv(self,atoms,zmat_variable,optgen,nbin=100,maxiter=100,
                        plot_opt=False):
       i,j           = zmat_variable
@@ -689,6 +731,7 @@ class AtomDance(object):
          plt.close() 
       return optlog
 
+
   def get_dz(self,atoms,zmat,dz,bin_):
       atoms_ = atoms.copy()
       atoms_ = self.zmat_to_cartation(atoms_,zmat)
@@ -702,6 +745,7 @@ class AtomDance(object):
       dEdz   = (e_-e)/bin_
       return dEdz
       
+
   def get_zmat_info(self,zmats):
       if zmats is None or len(zmats)==0:
          return None,0.0,0.0
@@ -732,6 +776,7 @@ class AtomDance(object):
                     return (i,j),lo,hi
       return None,0.0,0.0
 
+
   def get_zmats(self,mdtraj='md.traj'):
       images = Trajectory(mdtraj)
       mdzmat = []
@@ -741,6 +786,7 @@ class AtomDance(object):
       mdzmat = np.array(mdzmat)
       return mdzmat
 
+
   def check_bond(self,atoms=None,mdtraj=None,rmax=1.3):
       if atoms is None:
          atoms = self.ir.atoms
@@ -787,6 +833,7 @@ class AtomDance(object):
          bB_ += 1
       return bB_,bkbd
 
+
   def checkBond(self,bonds):
       newbond = False
       bd      = None
@@ -797,6 +844,7 @@ class AtomDance(object):
              return newbond,bd 
       return newbond,bd
 
+
   def check(self,wcheck=2,i=0,atoms=None,rmin=None):
       if atoms is None:
          atoms = self.ir.atoms
@@ -815,6 +863,7 @@ class AtomDance(object):
       fc.close()
       return atoms
 
+
   def checkLoneAtom(self,atoms,fc):
       for i in range(self.natom):
           if self.ir.Delta[i]<=self.ir.atol:
@@ -830,6 +879,7 @@ class AtomDance(object):
              self.ir.calculate_Delta(atoms)
       return atoms
 
+
   def checkLoneAtoms(self,atoms,fc):
       for i in range(self.natom):
           if self.ir.Delta[i]<=self.ir.atol:
@@ -886,6 +936,7 @@ class AtomDance(object):
              self.ir.calculate_Delta(atoms)
       return atoms
 
+
   def checkClosedAtom(self,atoms,fc):
       self.ir.calculate_Delta(atoms)
       neighbors = getNeighbor(self.natom,self.ir.r,self.ir.r_cuta,self.ir.bo0)
@@ -910,16 +961,19 @@ class AtomDance(object):
                  neighbors = getNeighbor(self.natom,self.ir.r,self.ir.r_cuta,self.ir.bo0)
       return atoms
 
+
   def bend(self,ang=None,rang=20.0,nbin=10,scale=1.2,traj='md.traj'):
       i,j,k = ang
       axis = [i,k]
       images = self.rotate(atms=[i,k],axis=axis,o=j,rang=rang,nbin=nbin,traj=traj,scale=scale)
       return images
 
+
   def bend_axis(self,axis=None,group=None,rang=20,nbin=30,scale=1.2,traj=None):
       images = self.rotate(atms=group,axis=axis,o=axis[0],rang=rang,nbin=nbin,traj=traj,scale=scale)
       return images
 
+
   def swing_group(self,ang=None,group=None,rang=20,nbin=30,scale=1.2,traj=None):
       i,j,k = ang
       atoms = self.ir.atoms
@@ -941,6 +995,7 @@ class AtomDance(object):
                             traj=traj,scale=scale)
       return images
 
+
   def rotate(self,atms=None,axis=None,axis_vector=None,o=None,rang=20.0,nbin=10,traj=None,scale=1.2):
       da = 2.0*rang/nbin
       atoms = self.ir.atoms
@@ -958,7 +1013,7 @@ class AtomDance(object):
          uk    = axis_vector
 
       a_     = -rang
-      for i in range(nbin):
+      for i in range(nbin+1):
           atoms_ = atoms.copy()
           for atomk in atms:
               vo  = atoms.positions[atomk] - atoms.positions[o] 
@@ -987,6 +1042,7 @@ class AtomDance(object):
       if not traj is None: his.close()
       return images
 
+
   def swing(self,ang,st=60.0,ed=180.0,nbin=50,scale=1.2,traj=None):
       da    = (ed - st)/nbin
       i,j,k = ang
@@ -1012,7 +1068,7 @@ class AtomDance(object):
       uj = np.cross(uk,ui)
       a_ = st
 
-      for i in range(nbin):
+      for i in range(nbin+1):
           atoms_ = atoms.copy()
           a = a_*3.141593/180.0
           p = r*np.cos(a)*ui + r*np.sin(a)*uj
@@ -1111,6 +1167,7 @@ class AtomDance(object):
        
       return images
   
+
   def stretch(self,pair,atoms=None,nbin=20,rst=0.7,red=1.3,scale=1.25,traj=None,
               ToBeMoved=None,neighbors=None):
       if atoms is None:
@@ -1138,7 +1195,7 @@ class AtomDance(object):
       bin_     = (red - rst)/nbin
       moveDirection = self.ir.vr[j][i]/self.ir.r[i][j]
 
-      for n in range(nbin):
+      for n in range(nbin+1):
           atoms_ = atoms.copy()
           moveV  = atoms.positions[i] + moveDirection*(rst+bin_*n)-atoms.positions[j]
           # print(self.ir.re[i][j]*self.rmin+bin_*n)
@@ -1164,6 +1221,7 @@ class AtomDance(object):
           if not traj is None: his.write(atoms=atoms_)
       return images
 
+
   def write_zmat(self,zmatrix,zfile=None):
       if zfile is None:
          zfile = self.label+'.zmat'
@@ -1175,6 +1233,7 @@ class AtomDance(object):
                      ' %7.4f,%8.4f,%9.4f ],' %(zmatrix[i][0],zmatrix[i][1],zmatrix[i][2]),
                      file=f)
 
+
   def close(self):
       self.ir        = None
       self.atom_name = None
