@@ -57,19 +57,19 @@ class LearningMachine(object):
                dt_mlmd=0.1, dt_aimd=0.3,dft_step=5,
                batch=50,max_batch=10,
                step=20000,md_step=20,MaxMDstep=500,colFrame=10,
-               mom_step=100,beta=0.777,
+               mom_step=100,
                mpopt=1, messages=1,
                optword='nocoul',
                T=300,Tmax=1000,
                label=None,
                convergence=0.01,nconvergence=3,
                accCriteria=0.95,lossCriteria=1000.0,lossTole=1.0,accTole=0.005,
-               accMax=0.96,accInc=1.0,mdInc=1.2,resetAcc=5,
-               learnWay=5,FreePairs=None,
+               accMax=0.96,accInc=1.0001,mdInc=1.2,resetAcc=1,
+               learnWay=5,FreePairs=None,beta=None,
                FreeAtoms=None,FirstAtom=None,
                rodic=None,
                rmin=0.6,rmax=1.3,angmax=15.0,
-               UseZmat=True,
+               CheckZmat=True,
                EngTole=0.05,dEtole=0.2,d2Etole=0.05,dEstop=2.0,
                nn=True,vdwnn=False,
                bo_layer=[2,1],mf_layer=[6,1],be_layer=[6,1],
@@ -78,7 +78,8 @@ class LearningMachine(object):
                bore={'others':0.45},
                weight={'others':2.0},
                spv_bm=False,spv_be=False,
-               bo_penalty=10000.0,
+               penalty=10000.0,
+               regularize=True,lambda_pen=0.000001,
                mefac=0.1,
                writelib=5000,
                xcf='VDW',xca='DRSLL',basistype='DZP'):
@@ -98,7 +99,7 @@ class LearningMachine(object):
       self.rmin         = rmin       # minimum bond length used to supervise ML-MD simulation
       self.rmax         = rmax
       self.angmax       = angmax
-      self.UseZmat      = UseZmat
+      self.CheckZmat    = CheckZmat
       self.convergence  = convergence
       self.nconvergence = nconvergence
       self.learnWay     = learnWay    # the way of learing
@@ -140,7 +141,9 @@ class LearningMachine(object):
       self.spv_be       = spv_be
       self.mefac        = mefac
       self.weight       = weight
-      self.bo_penalty   = bo_penalty
+      self.penalty      = penalty
+      self.regularize   = regularize
+      self.lambda_pen   = lambda_pen
       self.EnergyFunction = EnergyFunction
       self.MessageFunction= MessageFunction
       self.writelib     = writelib
@@ -157,7 +160,8 @@ class LearningMachine(object):
       self.natom        = self.a.natom
       self.atom_name    = self.a.atom_name
       self.get_ro(rodic)
-      self.get_fixatom()
+      # if self.freeatoms is None:
+      #    self.freeatoms = [i for i in range(self.natom)]
 
       if self.mpopt==1:
          self.messages  = 1
@@ -249,7 +253,9 @@ class LearningMachine(object):
              print('-  cannot find MD trajectory, use learnWay=1 in the first iter.')
              atoms = read(self.initConfig)
              learnWay = 1
-          
+          if not self.freeatoms is None:
+             atoms = self.a.check_momenta(atoms,freeatoms=self.freeatoms)
+             
           if learnWay in [4,5,6]:
              momsteps += mdsteps
              if momsteps >= mom_step:
@@ -276,8 +282,8 @@ class LearningMachine(object):
                       self.save_config()
                       if ms+1<self.natom:
                          j = self.a.zmat_id[ms+1]
-                         self.learnpair = [i,j]
-                         atoms = self.a.set_bond_momenta(i,j,atoms)
+                         # self.learnpair = [i,j]
+                         atoms,_,_ = self.a.set_bond_momenta(i,j,atoms)
                       else:
                          learnWay = 3
                          self.learnpair = None
@@ -295,7 +301,7 @@ class LearningMachine(object):
           else:
              dft_step = int(mdsteps/self.colFrame)+1
 
-          e_aimd,eml_,dEmax_,d2Emax_ = self.aimd(cwd,run_dir,dft_step,gen,learnWay)
+          e_aimd,eml_,dEmax_,d2Emax_,LabelDataLog = self.aimd(cwd,run_dir,dft_step,gen,learnWay)
           e_siesta.append(e_aimd[0])                                   # get siesta results              
                                                                        # start training  
           trajs_ = prep_data(label=self.label,direcs=data_dir,
@@ -328,7 +334,8 @@ class LearningMachine(object):
                                                      bore=self.bore, bom=self.bom,
                                                      spv_be=self.spv_be,spv_bm=self.spv_bm,
                                                      mefac=self.mefac,weight=self.weight,
-                                                     bo_penalty=self.bo_penalty)
+                                                     regularize=self.regularize,lambda_pen=self.lambda_pen,
+                                                     penalty=self.penalty)
                 self.load_config()
                 if tIter>=self.resetAcc and tstep>=self.step:
                    if accu>self.accCriteria-self.accTole and loss<self.lossCriteria+self.lossTole:
@@ -339,7 +346,7 @@ class LearningMachine(object):
                 tIter += 1
 
           self.lossCriteria = float(loss)
-          if self.UseZmat:
+          if self.CheckZmat:
              if not zmatopt is None:
                 optlog = self.a.get_optimal_zv(atoms,zmatopt,self.initConfig)
              if mdsteps>10:
@@ -366,7 +373,7 @@ class LearningMachine(object):
                 e_gmd,mdsteps,Deformed,zmats,zmat_variable,zvlo,zvhi = self.mlmd(cwd,run_dir,atoms, 10000,
                                                 iter_,learnWay,beta=self.beta,learnpair=self.learnpair,
                                                 groupi=groupi,groupj=groupj) 
-                # if self.UseZmat and mdsteps<=2:
+                # if self.CheckZmat and mdsteps<=2:
                 #    mdsteps= 7
                 #    images,relaxlog = self.a.zmat_relax(atoms=atoms,nbin=20,relax_step=mdsteps,traj='md.traj')
              if not images is None:
@@ -397,7 +404,8 @@ class LearningMachine(object):
           it.append(iter_)
 
           plot_energies(it,e_siesta,e_gulp)                             # plot learning status                                   
-          self.learninglog(iter_,loss,accu,mdsteps,e_gulp[-1],e_siesta[-1],dEmax_,d2Emax_,learnWay,Deformed,
+          self.learninglog(iter_,loss,accu,mdsteps,e_gulp[-1],e_siesta[-1],dEmax_,d2Emax_,LabelDataLog,
+          	               learnWay,Deformed,
                            self.a.InitZmat,unreasonable_zv,u_zvlo,u_zvhi,optlog,relaxlog)
           converg_ = converg
           if diff<=self.convergence: 
@@ -423,19 +431,35 @@ class LearningMachine(object):
              return diff
           iter_ += 1
 
-  def learninglog(self,i,loss,accu,mdsteps,eirff,edft,dEmax,d2Emax,learnWay,Deformed,
+  def learninglog(self,i,loss,accu,mdsteps,eirff,edft,dEmax,d2Emax,LabelDataLog,
+                  learnWay,Deformed,
                   InitZmat,uz,uzlo,uzhi,optlog,relaxlog):
       with open('learning.log','a') as l:
          l.write('                  ====================================                  \n')
-         l.write('                      Learning iteration:  %4d                         \n' %i)
+         l.write('                       Learning iteration:  {:4d}                        \n'.format(i))
          l.write('                  ====================================                  \n')
          l.write(' \n')
-         l.write('Loss of the model:  %f  \n' %loss)
-         l.write('Accuracy of the ML model:  %f  \n' %accu)
-         l.write('Steps of the ML MD:  %d  \n' %mdsteps)
-         l.write('Energy from ML and DFT:  %f  .vs.  %f \n' %(eirff,edft))
-         l.write('The max first derivative and second derivative:  %f,  %f \n' %(dEmax,d2Emax))
-         l.write('The learning method of this iter:  %d \n' %learnWay)
+         l.write('                     I&II  Labeling and Calling DFT ')
+         l.write('            Labeled atomic configurations and DFT calculations \n')
+         l.write(LabelDataLog)
+         # for d_,d in enumerate(LabeledData):
+         #     if d_%10==0:
+         #        l.write('\n')
+         #     l.write(' {:3d}'.format(d))
+         l.write(' \n')
+         l.write('                           III  Training ')
+         l.write('                          Train the model  \n')
+         l.write(' \n')
+         l.write('Loss of the model:  {:7.4f}  \n'.format(loss))
+         l.write('Accuracy of the potential model:  {:7.4f}  \n'.format(accu))
+         l.write('\n')
+         l.write('                           IV Exploring \n' %mdsteps)
+         l.write('            Exploring the configuration space by MD simulations \n' %mdsteps)
+         l.write('\n')
+         l.write('MD Steps:  {:d}  \n'.format(mdsteps))
+         l.write('Energy from ML and DFT:  {:f}  .vs.  {:f} \n'.format(eirff,edft))
+         l.write('The max first derivative and second derivative:  {:f}  {:f} \n'.format(dEmax,d2Emax))
+         l.write('The learning method of this iter:  {:d} \n'.format(learnWay))
          if not relaxlog is None:
             if Deformed>=1:
                l.write('Molecule structure is deformed, using zmatmix relax\n' )
@@ -464,7 +488,7 @@ class LearningMachine(object):
          popen('cp ../../md.traj ./')
          popen('cp ../../ffield.json ./')
          # print(' * files in this dir \n',listdir())
-         E,E_,dEmax,d2Emax = SinglePointEnergies('md.traj',label=self.label,EngTole=self.EngTole,
+         E,E_,dEmax,d2Emax,ind_ = SinglePointEnergies('md.traj',label=self.label,EngTole=self.EngTole,
                                                  frame=tstep,select=True,
                                                  dE=self.dEtole,d2E=self.d2Etole,
                                                  xcf=self.xcf,xca=self.xca,basistype=self.basistype,
@@ -474,7 +498,7 @@ class LearningMachine(object):
       else:
          raise RuntimeError('-  learn way not supported yet!')
       chdir(cwd)
-      return eaimd,emlmd,dEmax,d2Emax
+      return eaimd,emlmd,dEmax,d2Emax,ind_
 
   def mlmd(self,cwd,run_dir,atoms,Tmax,Iter,learnWay,learnpair=None,beta=None,
            groupi=None,groupj=None):        
@@ -483,10 +507,10 @@ class LearningMachine(object):
       irmd = IRMD(atoms=atoms,label=self.label,Iter=Iter,initT=self.T,
                   time_step=self.dt_mlmd,totstep=mdstep,Tmax=Tmax,
                   ro=self.ro,rmin=self.rmin,rmax=self.rmax,angmax=self.angmax,
-                  UseZmat=self.UseZmat,InitZmat=self.a.InitZmat,
+                  CheckZmat=self.CheckZmat,InitZmat=self.a.InitZmat,
                   zmat_id=self.a.zmat_id,zmat_index=self.a.zmat_index,
                   dEstop=self.dEstop,dEtole=self.dEtole,nn=self.nn,vdwnn=self.vdwnn,
-                  learnpair=learnpair,beta=beta,groupi=groupi,groupj=groupj)
+                  learnpair=learnpair,beta=beta,groupi=groupi,groupj=groupj,freeatoms=self.freeatoms)
       if learnWay==2:
          Deformed,zmats,zv,zvlo,zvhi = irmd.opt()
       else:
@@ -550,19 +574,19 @@ class LearningMachine(object):
                     'dEstop':self.dEstop,
                     'beta':self.beta,
                     'mdInc':self.mdInc,
-                    'dt_aimd':self.dt_aimd,
-                    'dt_mlmd':self.dt_mlmd,
+                    'regularize':self.regularize,
+                    'lambda_pen':self.lambda_pen,
+                    'penalty':self.penalty,
                     'rmin':self.rmin,
                     'Tmax':self.Tmax,
                     'T':self.T,
                     'maxiter':self.maxiter,
                     'max_batch':self.max_batch,
-                    'mpopt':self.mpopt,
                     'optword':self.optword,
                     'mefac':self.mefac,
                     'learnWay':self.learnWay,
                     'colFrame':self.colFrame,
-                    'UseZmat':self.UseZmat,
+                    'CheckZmat':self.CheckZmat,
                     'freeatoms':self.freeatoms,
                     'freepairs':self.freepairs,
                     'bore':self.bore,
@@ -592,19 +616,19 @@ class LearningMachine(object):
            self.dEstop       = options['dEstop'] 
            self.beta         = options['beta'] 
            self.mdInc        = options['mdInc'] 
-           self.dt_aimd      = options['dt_aimd'] 
-           self.dt_mlmd      = options['dt_mlmd'] 
+           self.regularize   = options['regularize'] 
+           self.lambda_pen   = options['lambda_pen'] 
+           self.penalty      = options['penalty']
            self.rmin         = options['rmin'] 
            self.Tmax         = options['Tmax'] 
            self.T            = options['T'] 
            self.maxiter      = options['maxiter']
            self.max_batch    = options['max_batch']
-           self.mpopt        = options['mpopt']
            self.optword      = options['optword']
            self.mefac        = options['mefac']
            self.learnWay     = options['learnWay']
            self.colFrame     = options['colFrame']
-           self.UseZmat      = options['UseZmat']
+           self.CheckZmat    = options['CheckZmat']
            self.bore         = options['bore']
            self.writelib     = options['writelib']
            self.freepairs    = options['freepairs']
@@ -635,7 +659,7 @@ if __name__ == '__main__':
                         convergence=0.04,accCriteria=0.9,accMax=0.962,
                         accTole=0.01,accInc=1.001,
                         lossCriteria=26.0,lossTole=2.0,
-                        UseZmat=False,
+                        CheckZmat=False,
                         resetAcc=1,
                         mdInc=1.1,
                         nn=True,
