@@ -33,6 +33,8 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
 
     for i,atoms in enumerate(images):
         energy = atoms.get_potential_energy()
+        extreme_point = False
+
         if i>0: 
            if i<(tframe-1):
               deltEl = energy - energies[-1]
@@ -41,17 +43,21 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
               d2E_= abs(deltEr-deltEl)
            else:
               deltEl =  energy - energies[-1]
+              deltEr =  deltEl
               dE_ = abs(deltEl)
-        
-        energies.append(energy)
+
+           if (deltEr>0.0 and deltEl<0.0) or (deltEr<0.0 and deltEl>0.0) or dE_<0.00001:
+              extreme_point = True
+
+           if select:
+              if dE_>dE or extreme_point:
+                 if i not in ind_:
+                    if i-1 not in ind_:
+                       ind_.append(i)
+
         dEs.append(dE_)
         d2Es.append(d2E_)
-
-        if select:
-           if dE_>dE or d2E_>d2E:
-              if i not in ind_:
-                 if i-1 not in ind_:
-                    ind_.append(i)
+        energies.append(energy)
 
     ide  = np.argmax(dEs)
     id2e = np.argmax(d2Es)
@@ -61,6 +67,9 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
        ind_.append(id2e)
     ind_.sort()
 
+    LabelDataLog = 'AtomicConfigure   E(ML)   E(DFT)   Diff    dE   d2E  \n'
+    LabelDataLog = '-----------------------------------------------------\n'
+
     for i in ind_:
         atoms = images[i]
         e_    = atoms.get_potential_energy()
@@ -73,9 +82,8 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
         E_.append(e_)
 
         diff_ = abs(e-e_)
-
-        print(' * %d Energies from MLP: %9.5f DFT: %9.5f Diff: %6.6f dE: %5.4f d2E: %5.4f' %(i,
-                      e_,e,diff_,dE_,d2E_))
+        LabelDataLog += '   {:3d}   {:9.5f}  {:9.5f}  {:6.6f}  {:5.4f}   {:5.4f}\n'.format(i,
+                         e_,e,diff_,dE_,d2E_)
         with open('SinglePointEnergies.log','a') as fs:  
              fs.write('%d MLP: %9.5f DFT: %9.5f Diff: %6.6f dE: %5.4f d2E: %5.4f\n' %(i,
                       e_,e,diff_,dE_,d2E_))
@@ -87,5 +95,5 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
     images = None
     dEmax  = dEs[ide]
     d2Emax = d2Es[id2e]
-    return E,E_,dEmax,d2Emax
+    return E,E_,dEmax,d2Emax,LabelDataLog
 
