@@ -102,13 +102,13 @@ class Init_Check(object):
              # if p[key]<=12.0:
              #    p[key] = value(16.0,key)
 
-          if k=='lp2':
-             sp = key.split('_')[1]
-             if p['val_'+sp]!=p['vale_'+sp]:
-                if p[key] <= 0.001:
-                   p[key] = value(4.00,key)
-             if p[key]>=60.0:
-                p[key] = value(20.0,key)
+          # if k=='lp2':
+          #    sp = key.split('_')[1]
+          #    if p['val_'+sp]!=p['vale_'+sp]:
+          #       if p[key] <= 0.001:
+          #          p[key] = value(4.00,key)
+          #    if p[key]>=60.0:
+          #       p[key] = value(20.0,key)
 
           if k=='coa1': 
              if p[key]<= -39.0:
@@ -256,9 +256,9 @@ class Init_Check(object):
           if k=='val7': 
              if p[key]>=20.0:
                 p[key] = value(18.0,key)
-          if k=='ovun1': 
-             if p[key]<=0.100:
-                p[key]= value(0.12,key)
+          # if k=='ovun1': 
+          #    if p[key]<=0.100:
+          #       p[key]= value(0.12,key)
           if k=='ovun2': 
              if p[key]>=0.0:
                 p[key]= value(-3.0,key)
@@ -270,10 +270,8 @@ class Init_Check(object):
              if p[key]>=20.00:
                 p[key]= value(2.0,key)
           if k=='ovun5': 
-             if p[key]>=90.0:
-                p[key]= value(49.0,key)
-             if p[key]<=0.001:
-                p[key]= value(0.001,key)
+             if p[key]>=60.0:
+                p[key]= value(59.0,key)
           if k in ['bo2','bo4','bo6']:     ### scale bo
              if p[key]>=8.0:
                 p[key] = value(7.95,key)
@@ -359,27 +357,28 @@ class Init_Check(object):
 
   def check_m(self,m):
       ''' reset the dead neuron '''
-      # for key in m:
-      #     k = key.split('_')[0]
-      #     for i,mmm in enumerate(m[key]):
-      #         if isinstance(mmm, list):
-      #            for j,mm in enumerate(mmm):
-      #                if isinstance(mm, list):
-      #                   for l,m_ in enumerate(mm):
-      #                       if m[key][i][j][l]>=16.0:
-      #                          m[key][i][j][l] = 2.0 
-      #                       if m[key][i][j][l]<=-16.0:
-      #                          m[key][i][j][l]= -2.0 
-      #                else:
-      #                   if m[key][i][j]>=16.0:
-      #                      m[key][i][j] = 2.0 
-      #                   if m[key][i][j]<=-16.0:
-      #                      m[key][i][j]= -2.0 
-      #         else:
-      #            if m[key][i]>=16.0:
-      #               m[key][i] = 2.0 
-      #            if m[key][i]<=-16.0:
-      #               m[key][i] = -2.0 
+      print('-  check neurons that have big values ...')
+      for key in m:
+          k = key.split('_')[0]
+          for i,mmm in enumerate(m[key]):
+              if isinstance(mmm, list):
+                 for j,mm in enumerate(mmm):
+                     if isinstance(mm, list):
+                        for l,m_ in enumerate(mm):
+                            if m[key][i][j][l]>=16.0:
+                               m[key][i][j][l] = 2.0 
+                            if m[key][i][j][l]<=-16.0:
+                               m[key][i][j][l]= -2.0 
+                     else:
+                        if m[key][i][j]>=16.0:
+                           m[key][i][j] = 2.0 
+                        if m[key][i][j]<=-16.0:
+                           m[key][i][j]= -2.0 
+              else:
+                 if m[key][i]>=16.0:
+                    m[key][i] = 2.0 
+                 if m[key][i]<=-16.0:
+                    m[key][i] = -2.0 
       return m
 
 
