@@ -335,7 +335,7 @@ def nvt(atoms=None,gen='poscar.gen',T=350,time_step=0.1,tot_step=100,index=-1,mo
                   time_step=time_step,
                   tot_step=tot_step,
                   lib=lib)
-    print('\n-  running gulp nvt ...')
+    # print('\n-  running gulp nvt ...')
     system('gulp<inp-gulp>gulp.out')
     xyztotraj('his.xyz',mode=mode)
 
