@@ -66,8 +66,8 @@ class IRMD(object):
                index=-1,totstep=100,vdwnn=False,nn=True,
                initT=300,Tmax=10000,time_step=0.1,Iter=0,
                ro=None,rmin=0.6,rmax=1.3,angmax=20.0,
-               UseZmat=False,zmat_id=None,zmat_index=None,InitZmat=None,
-               learnpair=None,groupi=[],groupj=[],beta=None,
+               CheckZmat=False,zmat_id=None,zmat_index=None,InitZmat=None,
+               learnpair=None,groupi=[],groupj=[],beta=None,freeatoms=None,
                dEstop=1000,dEtole=1.0,print_interval=1):
       self.Epot      = []
       self.epot      = 0.0
@@ -84,7 +84,7 @@ class IRMD(object):
       self.step      = 0
       self.rmax      = rmax
       self.angmax    = angmax
-      self.UseZmat   = UseZmat
+      self.CheckZmat = CheckZmat
       self.zmat_id   = zmat_id
       self.zmat_index= zmat_index
       self.InitZmat  = InitZmat
@@ -95,6 +95,7 @@ class IRMD(object):
       self.groupi    = groupi
       self.groupj    = groupj
       self.beta      = beta
+      self.freeatoms = freeatoms
       self.print_interval = print_interval
 
       if self.atoms is None:
@@ -159,11 +160,15 @@ class IRMD(object):
                     vd  = d*vij
                     vd_ = d_*vij
                     v[iv] = vd_ + self.beta*(v[iv]-vd)
-             else:
-                v = v*self.beta
+             elif not self.freeatoms is None:
+                for iv,v_ in enumerate(v):
+                    if iv not in self.freeatoms:
+                       v[iv] = v_*self.beta
+             # else:
+             #    v = v*self.beta
              self.atoms.set_velocities(v)
 
-          if self.UseZmat:
+          if self.CheckZmat:
              self.Deformed,zmat,self.zv,self.zvlo,self.zvhi = check_zmat(atoms=a,rmin=self.rmin,rmax=self.rmax,
                                              angmax=self.angmax,zmat_id=self.zmat_id,
                                              zmat_index=self.zmat_index,InitZmat=self.InitZmat)
@@ -225,7 +230,7 @@ class IRMD(object):
                 'Etot = %.3feV' % (self.step,self.epot,self.ekin,self.T,
                                    self.epot + self.ekin))
           try:
-             if self.UseZmat:
+             if self.CheckZmat:
                 assert self.Deformed<1 and self.dEstop_<self.dEstop,'Atoms structure is deformed!' 
              else:
                 assert n==0 and self.T<self.Tmax and self.dEstop_<self.dEstop,'Atoms too closed or Temperature goes too high!' 
@@ -256,7 +261,7 @@ class IRMD(object):
       def check(a=self.atoms):
           epot_      = a.get_potential_energy()
           n,self.Deformed = 0,0
-          if self.UseZmat:
+          if self.CheckZmat:
              Deformed,zmat,self.zv,self.zvlo,self.zvhi = check_zmat(atoms=a,rmin=self.rmin,rmax=self.rmax,angmax=self.angmax,
                                         zmat_id=self.zmat_id,zmat_index=self.zmat_index,
                                         InitZmat=self.InitZmat)
@@ -294,7 +299,7 @@ class IRMD(object):
 
           self.step  = self.dyn.nsteps
           try:
-             if self.UseZmat:
+             if self.CheckZmat:
                 assert Deformed<1,'Atoms too closed or Deformed!' 
              else:
                 assert n==0 and (not np.isnan(epot_)),'Atoms too closed!'
