@@ -22,15 +22,20 @@ import json as js
 tf.compat.v1.disable_eager_execution()
 
 
+def dsigmoid(x):
+    ds = tf.sigmoid(x)(1-tf.sigmoid(x))
+    return ds
+
 
 class MPNN(ReaxFF):
   def __init__(self,libfile='ffield',direcs={},
                dft='ase',atoms=None,
                cons=['val','vale',
-                     # 'rosi','ropi','ropp', 
-                     # 'ovun3', # 'val8','val9','val10',
-                     # 'bo1','bo2','bo3','bo4','bo5','bo6',
-                     'Depi','Depp','lp3','cutoff'], # 'hbtol'
+                     # 'ovun1','ovun2','ovun3','ovun4',
+                     'ovun5','ovun6','ovun7','ovun8',
+                     'lp1','lp2','lp3',
+                     'pen1','pen2','pen3','pen4',
+                     'Depi','Depp','cutoff','acut'], # 'hbtol'
                optmol=True,mefac=0.1,
                opt=None,optword='nocoul',
                nanv={'boc1':-2.0},
@@ -38,7 +43,9 @@ class MPNN(ReaxFF):
                hbshort=6.75,hblong=7.5,
                vdwcut=10.0,
                bore={'C-C':0.5,'others':0.45},
-               bom={'others':1.05},
+               bom={'others':1.2},
+               spv_bm=False,
+               spv_be=False,
                weight={'others':1.0},
                ro_scale=0.1,
                clip_op=True,
@@ -48,8 +55,8 @@ class MPNN(ReaxFF):
                messages=1,
                TwoBodyOnly=False,
                mpopt=None,
-               bo_layer=[2,1],
-               mf_layer=[6,1],
+               bo_layer=[4,1],
+               mf_layer=[9,2],
                be_layer=[6,1],
                vdw_layer=[4,1],
                vdwnn=False,
@@ -58,9 +65,9 @@ class MPNN(ReaxFF):
                spec=[],
                sort=False,
                pkl=False,
-               spv_bm=False,
-               spv_be=False,
-               bo_penalty=100000.0,
+               penalty=100000.0,
+               lambda_pen=0.000001,
+               regularize=False,
                optMethod='ADAM',
                maxstep=60000,
                emse=0.9,
@@ -71,7 +78,7 @@ class MPNN(ReaxFF):
                huber_d=30.0,
                ncpu=None):
       '''
-         Massage Passing Neural network build top on ReaxFF potentail
+         Message Passing Neural network build top on ReaxFF potentail
          version 3.0 
            Time: 2018-10-20
            Intelligence ReaxFF Neual Network: Evoluting the Force Field parameters on-the-fly
@@ -84,7 +91,8 @@ class MPNN(ReaxFF):
       self.spv_be           = spv_be
       self.spv_bm           = spv_bm
       self.TwoBodyOnly      = TwoBodyOnly
-      # self.bo_layer       = bo_layer
+      self.regularize       = regularize 
+      self.lambda_pen       = lambda_pen
       self.mf_layer         = mf_layer
       self.be_layer         = be_layer
       self.vdw_layer        = vdw_layer
@@ -102,7 +110,7 @@ class MPNN(ReaxFF):
                       ro_scale=ro_scale,
                       clip_op=clip_op,InitCheck=InitCheck,resetDeadNeuron=resetDeadNeuron,nn=nn,
                       bo_layer=bo_layer,spec=spec,sort=sort,pkl=pkl,weight=weight,
-                      bore=bore,bo_penalty=bo_penalty,
+                      bore=bore,penalty=penalty,
                       optMethod=optMethod,maxstep=maxstep,
                       emse=emse,convergence=convergence,lossConvergence=lossConvergence,
                       losFunc=losFunc,conf_vale=conf_vale,
@@ -132,14 +140,13 @@ class MPNN(ReaxFF):
              else:
                 bm_ = self.bom['others']
              
-             # if not self.TwoBodyOnly:
              fao = tf.where(tf.greater(self.rbd[bd],self.rcuta[bd]),1.0,0.0)    ##### r> rcuta that bo = 0.0
-             self.diffn[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu(self.bo0[bd]*fao-self.botol)) # self.atol
+             self.diffn[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu(self.bo0[bd]*fao - self.botol*fao))
              l_atol = tf.add(self.diffn[bd],l_atol)
 
              if self.spv_bm:
-                fao_= tf.where(tf.less_equal(self.rbd[bd],bm_*self.re[bd]),1.0,0.0) ##### r< 1.2*re that bo > botol
-                self.diffa[bd] = tf.reduce_sum(input_tensor=tf.nn.relu(self.botol-self.bo0[bd])*fao_)
+                fao_= tf.where(tf.less_equal(self.rbd[bd],bm_*self.re[bd]),1.0,0.0) ##### r< 1.2*r_e that bo > botol
+                self.diffa[bd] = tf.reduce_sum(input_tensor=tf.nn.relu((self.botol-self.bo0[bd])*fao_))
                 l_atol = tf.add(self.diffa[bd],l_atol)
 
              #  bop should be zero if r>rcut
@@ -148,17 +155,46 @@ class MPNN(ReaxFF):
              l_atol  = tf.add(self.diffb[bd],l_atol)
              
              if self.spv_be:
-                fe  = tf.where(tf.less_equal(self.rbd[bd],self.re[bd]),1.0,0.0) ##### r< 0.9*re that bo > bore_
+                fe  = tf.where(tf.less_equal(self.rbd[bd],self.re[bd]),1.0,0.0) ##### r< r_e that bo > bore_
                 self.diffe[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((bore_-self.bo0[bd])*fe))
                 l_atol  = tf.add(self.diffe[bd],l_atol)
+            
+                fe  = tf.where(tf.less_equal(self.rbd[bd],self.re[bd]),1.0,0.0) ##### r< r_e that bo > bore_
+                self.diffe[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((0.2*self.p['Desi_'+bd]+self.EBD[bd])*fe))
+                l_atol  = tf.add(self.diffe[bd],l_atol)
 
              if self.EnergyFunction == 1:
-                fe  = tf.where(tf.less_equal(self.bo0[bd],self.botol),1.0,0.0) ##### bo <= 0.0*re that e = 0.0
-                self.diffesi[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((self.EBD[bd])*fe))
+                fe  = tf.where(tf.less_equal(self.bo0[bd],self.botol),1.0,0.0) ##### bo <= 0.0 that e = 0.0
+                self.diffesi[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((-self.EBD[bd])*fe))
                 l_atol  = tf.add(self.diffesi[bd],l_atol)
 
+             # penalize term for regularization of the neural networs
+             if self.regularize:                                               # regularize
+                pen_bo_w  = tf.reduce_sum(tf.square(self.m['fsiwi_'+bd]))
+                pen_bo_w += tf.reduce_sum(tf.square(self.m['fpiwi_'+bd]))
+                pen_bo_w += tf.reduce_sum(tf.square(self.m['fppwi_'+bd]))
+                for l in range(self.bo_layer[1]):                                               
+                    pen_bo_w += tf.reduce_sum(tf.square(self.m['fsiw_'+bd][l]))
+                    pen_bo_w += tf.reduce_sum(tf.square(self.m['fpiw_'+bd][l]))
+                    pen_bo_w += tf.reduce_sum(tf.square(self.m['fppw_'+bd][l]))
+                pen_bo_w += tf.reduce_sum(tf.square(self.m['fsiwo_'+bd])) 
+                pen_bo_w += tf.reduce_sum(tf.square(self.m['fpiwo_'+bd])) 
+                pen_bo_w += tf.reduce_sum(tf.square(self.m['fppwo_'+bd])) 
+
+                for t in range(1,self.messages+1):
+                    pen_bo_w  += tf.reduce_sum(tf.square(self.m['f'+str(t)+'wi_'+bd]))
+                    for l in range(self.mf_layer[1]):                                               
+                        pen_bo_w += tf.reduce_sum(tf.square(self.m['f'+str(t)+'w_'+bd][l]))
+                    pen_bo_w += tf.reduce_sum(tf.square(self.m['f'+str(t)+'wo_'+bd])) 
+
+                pen_bo_w  = tf.reduce_sum(tf.square(self.m['fewi_'+bd]))
+                for l in range(self.be_layer[1]):                                               
+                    pen_bo_w += tf.reduce_sum(tf.square(self.m['few_'+bd][l]))
+                pen_bo_w += tf.reduce_sum(tf.square(self.m['fewo_'+bd])) 
+                l_atol= tf.add(self.lambda_pen*pen_bo_w,l_atol)
+
              l_atol= tf.add(tf.nn.relu(self.rc_bo[bd]-self.rcut[bd]),l_atol)
-      return l_atol*self.bo_penalty
+      return l_atol*self.penalty
 
   def get_loss(self):
       self.Loss = 0.0
@@ -233,6 +269,19 @@ class MPNN(ReaxFF):
           zpe_ = tf.gather_nd(self.EATOM,self.atomlink[mol]) 
           self.zpe[mol] = tf.reduce_sum(input_tensor=zpe_,name='zpe') + self.MolEnergy[mols]
 
+  # def get_elone(self,atom,D):
+  #     NLPOPT              = 0.5*(self.p['vale_'+atom] - self.p['val_'+atom])
+  #     self.Delta_e[atom]  = 0.5*(self.p['vale_'+atom] - D)
+
+  #     # self.DE[atom]     = -tf.nn.relu(-tf.math.ceil(self.Delta_e[atom])) 
+  #     # self.nlp[atom]    = -self.DE[atom] + tf.exp(-self.p['lp1']*4.0*tf.square(1.0+self.Delta_e[atom]-self.DE[atom]))
+  #     self.nlp[atom]      = self.Delta_e[atom]
+  #     self.Delta_lp[atom] = NLPOPT- self.Delta_e[atom]                     # nan error
+      
+  #     self.explp[atom]    = 1.0+tf.exp(-self.p['lp3']*self.Delta_lp[atom])
+
+  #     self.EL[atom] = tf.math.divide(self.p['lp2_'+atom]*self.Delta_lp[atom],self.explp[atom],
+  #                                    name='Elone_%s' %atom)
 
   def get_total_energy(self):
       for mol in self.mols:
@@ -260,6 +309,7 @@ class MPNN(ReaxFF):
                                   self.eself[mol], 
                                   self.zpe[mol],name='E_%s' %mol)  
 
+
   def f_nn(self,pre,bd,nbd,x,layer=5):
       ''' Dimention: (nbatch,4) input = 4
                  Wi:  (4,8) 
@@ -292,7 +342,7 @@ class MPNN(ReaxFF):
               tf.math.divide(20.0,tf.pow(self.vdwcut,7.0))*tf.pow(r,7.0)
       return tp
 
-  def fmassage(self,pre,bd,nbd,x,layer=5):
+  def fmessage(self,pre,bd,nbd,x,layer=5):
       ''' Dimention: (nbatch,4) input = 4
                  Wi:  (4,8) 
                  Wh:  (8,8)
@@ -359,8 +409,8 @@ class MPNN(ReaxFF):
          bopi  = self.Hpi[t-1][bd]*F
          bopp  = self.Hpp[t-1][bd]*F
       elif self.MessageFunction==2:
-         Fi    = self.fmassage(flabel,bd, self.nbd[bd],[Dbi,Dbj,h],layer=self.mf_layer[1])
-         Fj    = self.fmassage(flabel,bdr,self.nbd[bd],[Dbj,Dbi,h],layer=self.mf_layer[1])
+         Fi    = self.fmessage(flabel,bd, self.nbd[bd],[Dbi,Dbj,h],layer=self.mf_layer[1])
+         Fj    = self.fmessage(flabel,bdr,self.nbd[bd],[Dbj,Dbi,h],layer=self.mf_layer[1])
          F     = Fi*Fj
          Fsi,Fpi,Fpp = tf.unstack(F,axis=2)
        
@@ -407,7 +457,7 @@ class MPNN(ReaxFF):
           self.ebond[mol]= tf.reduce_sum(input_tensor=self.ebda[mol],axis=0,name='bondenergy')
 
   def message_passing(self):
-      ''' finding the final Bond－order with a massage passing '''
+      ''' finding the final Bond－order with a message passing '''
       self.H.append(self.bop)                   # 
       self.Hsi.append(self.bop_si)              #
       self.Hpi.append(self.bop_pi)              #
