@@ -114,7 +114,7 @@ def DIV_IF(y,x):
 class ReaxFF(object):
   def __init__(self,libfile='ffield',direcs={},
                dft='ase',atoms=None,
-               cons=['val','vale','lp3'], #,'hbtol'
+               cons=['val','vale','lp3','cutoff','acut'], #,'hbtol'
                opt=None,optword='nocoul',
                VariablesToOpt=None,
                nanv={'boc1':-2.0},
@@ -139,7 +139,7 @@ class ReaxFF(object):
                popSize=500,
                fromPop=False,
                board=False,
-               bo_penalty=100000.0,
+               penalty=100000.0,
                to_train=True,
                optMethod='ADAM',
                maxstep=60000,
@@ -209,7 +209,7 @@ class ReaxFF(object):
          self.p_,self.m_ = self.ic.check(self.p_,self.m_,resetDeadNeuron=self.resetDeadNeuron)
 
       self.set_neurons()
-      self.bo_penalty    = bo_penalty
+      self.penalty       = penalty
       self.logger        = logger('training.log')
       self.initialized   = False  
 
@@ -551,7 +551,7 @@ class ReaxFF(object):
              
              l_atol = tf.add(self.diffe[bd],l_atol)
              l_atol = tf.add(tf.nn.relu(self.rc_bo[bd]-self.rcut[bd]),l_atol)
-      return l_atol*self.bo_penalty
+      return l_atol*self.penalty
 
 
   def get_bond_energy(self):
@@ -617,11 +617,10 @@ class ReaxFF(object):
       self.powb[bd] = tf.pow(self.bosi[bd]+FBOR,self.p['be2_'+bd])
       self.expb[bd] = tf.exp(tf.multiply(self.p['be1_'+bd],1.0-self.powb[bd]))
 
-      self.sieng[bd]= self.p['Desi_'+bd]*self.bosi[bd]*self.expb[bd]*FBO 
-      self.pieng[bd]= tf.multiply(self.p['Depi_'+bd],self.bopi[bd])
-      self.ppeng[bd]= tf.multiply(self.p['Depp_'+bd],self.bopp[bd]) 
-
-      self.EBD[bd]  = - self.sieng[bd] - self.pieng[bd] - self.ppeng[bd]
+      self.sieng[bd] = self.p['Desi_'+bd]*self.bosi[bd]*self.expb[bd]*FBO 
+      self.pieng[bd] = tf.multiply(self.p['Depi_'+bd],self.bopi[bd])
+      self.ppeng[bd] = tf.multiply(self.p['Depp_'+bd],self.bopp[bd]) 
+      self.EBD[bd]   = - self.sieng[bd] - self.pieng[bd] - self.ppeng[bd]
 
 
   def get_atom_energy(self):
@@ -683,7 +682,6 @@ class ReaxFF(object):
       NLPOPT             = 0.5*(self.p['vale_'+atom] - self.p['val_'+atom])
       self.Delta_e[atom] = 0.5*(D - self.p['vale_'+atom])
       self.DE[atom]      = -tf.nn.relu(-tf.math.ceil(self.Delta_e[atom])) 
-
       self.nlp[atom]     = -self.DE[atom] + tf.exp(-self.p['lp1']*4.0*tf.square(1.0+self.Delta_e[atom]-self.DE[atom]))
       
       self.Delta_lp[atom] = NLPOPT-self.nlp[atom]                                        # nan error
@@ -1278,15 +1276,15 @@ class ReaxFF(object):
           elif key=='atomic':
              self.p[k] = tf.clip_by_value(self.v[k],-99.9999,999.9999)
           elif key=='acut':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0001,0.001)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0001,0.0002)
           elif key=='hbtol':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0001,0.001)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0001,0.0002)
           elif key=='cutoff':
              self.p[k] = tf.clip_by_value(self.v[k],0.01,0.012)
           elif key=='ovun5':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,60.0*self.unit)
           elif key=='ovun1':
-             self.p[k] = tf.clip_by_value(self.v[k],0.1,2.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,2.0)
           elif key in ['boc4','boc5','vdw1']:
              self.p[k] = tf.clip_by_value(self.v[k],0.00000001,100.0)
           elif key == 'boc3':
@@ -1302,11 +1300,11 @@ class ReaxFF(object):
           elif key == 'Dehb':
              self.p[k] = tf.clip_by_value(self.v[k],-3.0*self.unit,3.0*self.unit)
           elif key == 'Desi':
-             self.p[k] = tf.clip_by_value(self.v[k],60.000*self.unit,999.0*self.unit)
+             self.p[k] = tf.clip_by_value(self.v[k],100.000*self.unit,699.0*self.unit)
           elif key== 'Depi':
-             self.p[k] = tf.clip_by_value(self.v[k],0.000*self.unit,999.0*self.unit)
+             self.p[k] = tf.clip_by_value(self.v[k],0.000*self.unit,299.0*self.unit)
           elif key== 'Depp':
-             self.p[k] = tf.clip_by_value(self.v[k], 0.000*self.unit,999.0*self.unit)
+             self.p[k] = tf.clip_by_value(self.v[k], 0.000*self.unit,299.0*self.unit)
           elif key=='tor1':
              self.p[k] = tf.clip_by_value(self.v[k],-99.9900,-0.0)
           elif key== 'ovun2':
@@ -1324,12 +1322,22 @@ class ReaxFF(object):
              self.p[k] = tf.clip_by_value(self.v[k],-99.9900*self.unit,990.0*self.unit)
           elif key in ['bo1','bo3','bo5']:
              if self.nn:
-                self.p[k] = tf.clip_by_value(self.v[k],-0.9,-0.1)
+                if key =='bo1':
+                   self.p[k] = tf.clip_by_value(self.v[k],-0.2,-0.15)
+                elif key =='bo3':
+                   self.p[k] = tf.clip_by_value(self.v[k],-0.15,-0.1)
+                elif key =='bo5':
+                   self.p[k] = tf.clip_by_value(self.v[k],-0.15,-0.1)
              else:
                 self.p[k] = tf.clip_by_value(self.v[k],-50.0,-0.0001)
           elif key in ['bo2','bo4','bo6']:
              if self.nn:
-                self.p[k] = tf.clip_by_value(self.v[k],2.0,8.0)
+                if key =='bo2':
+                   self.p[k] = tf.clip_by_value(self.v[k],6.0,7.0)
+                elif key =='bo4':
+                   self.p[k] = tf.clip_by_value(self.v[k],5.0,6.0)
+                elif key =='bo6':
+                   self.p[k] = tf.clip_by_value(self.v[k],4.0,5.0)
              else:
                 self.p[k] = tf.clip_by_value(self.v[k],0.0,50.0)
           elif key == 'boc1':
@@ -1341,7 +1349,7 @@ class ReaxFF(object):
           elif key== 'lp1':
              self.p[k] = tf.clip_by_value(self.v[k],10.0,50.0)
           elif key== 'lp2':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0,99.0*self.unit)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,39.0*self.unit)
           elif key== 'lp3':
              self.p[k] = tf.clip_by_value(self.v[k],10.0,75.0001)
           elif key == 'be1':
@@ -1383,7 +1391,7 @@ class ReaxFF(object):
              self.p[k] = tf.clip_by_value(self.v[k],0.5*self.p_['val_'+sp],2.0*self.p_['val_'+sp])
           else:
              self.p[k] = self.v[k]
-  
+
 
   def set_parameters_ud(self,opt=[],libfile=None):
       if not libfile is None:
@@ -1663,29 +1671,6 @@ class ReaxFF(object):
       self.feed_dict = self.feed_data()   
 
 
-  def sa(self,total_step=10000,step=5000,astep=5000,bstep=1000,
-          print_step=10,writelib=1000):
-      ''' simulated anealing auto train '''
-      for i in range(total_step):
-          self.nnopt= True
-          if i!=0: self.reset(opt=self.opt)
-          self.logger.info('-  optiming all variables ...')
-          loss,accu,accMax,i = self.run(learning_rate=1.0e-4,method='AdamOptimizer',
-                      step=step,print_step=print_step,writelib=writelib)
-          if np.isnan(loss):
-             print('-  Job continue, the loss is NaN.')
-             self.p_  = self.ic.auto(self.p_)
-          
-          self.nnopt  = False
-          self.reset(opt=self.angopt+self.toropt+self.lopt)
-          self.logger.info('-  optiming angle and torsion variables ...')
-          loss,accu,accMax,i = self.run(learning_rate=1.0e-3,method='AdamOptimizer',
-                      step=astep,print_step=print_step,writelib=writelib)
-          if np.isnan(loss):
-             print('-  Job continue, the loss is NaN.')
-             self.p_  = self.ic.auto(self.p_)
-
-
   def run(self,learning_rate=1.0e-4,method='AdamOptimizer',
                step=2000,print_step=10,writelib=100):
       if not self.initialized:
