@@ -15,7 +15,7 @@ from .reaxfflib import read_lib,write_lib
 from .qeq import qeq
 import random
 import pickle
-np.set_printoptions(threshold=np.inf) 
+# np.set_printoptions(threshold=np.inf) 
 
 
 def rtaper(r,rmin=6.75,rmax=7.5):
