@@ -30,7 +30,7 @@ def train(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence=1000
           bore={'others':0.45},
           bom={'others':1.20},
           weight={'others':10.0},
-          bo_penalty=10000.0,
+          penalty=10000.0,
           ffield = 'ffield.json'):
     ''' training the force field '''
     rn = ReaxFF(libfile=ffield,
@@ -43,7 +43,7 @@ def train(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence=1000
                 pkl=False,
                 batch_size=batch,
                 losFunc='n2',
-                bo_penalty=10000.0,
+                penalty=10000.0,
                 nanv=nanv,
                 weight=weight,
                 bore=bore,
@@ -81,24 +81,23 @@ def train(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence=1000
 
 
 def train_mpnn(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence=1000.0,
-          nanv={'pen3':-2.0,'pen4':-2.0,'tor2':-5.0,'tor4':-5.0,
-                'ovun3':-2.0,'ovun4':-2.0,
-                'lp1':-3.0,'vdw1':-3.0},
-          optword='nocoul',
-          spec=[],
-          writelib=1000,nn=True,vdwnn=False,
-          bo_layer=[3,1],
-          mf_layer=[9,2],
-          be_layer=[3,1],
-          EnergyFunction=3,MessageFunction=2,mpopt=3,
-          bore={'others':0.45},
-          bom={'others':1.0},
-          spv_bm=False,
-          spv_be=False,
-          mefac=0.1,
-          weight={'others':10.0},
-          bo_penalty=10000.0,
-          ffield = 'ffield.json'):
+               nanv={'pen3':-2.0,'pen4':-2.0,'tor2':-5.0,'tor4':-5.0,
+                     'ovun3':-2.0,'ovun4':-2.0,
+                     'lp1':-3.0,'vdw1':-3.0},
+               optword='nocoul',
+               spec=[],
+               writelib=1000,nn=True,vdwnn=False,
+               bo_layer=[3,1],
+               mf_layer=[9,2],
+               be_layer=[3,1],
+               EnergyFunction=3,MessageFunction=2,mpopt=3,
+               bore={'others':0.45},spv_be=False,
+               bom={'others':1.0},spv_bm=False,
+               mefac=0.1,
+               weight={'others':10.0},
+               regularize=True,lambda_pen=0.0001,
+               penalty=100.0,
+               ffield = 'ffield.json'):
     ''' train the massage passing model '''
     opt_ = None
     if mpopt==1:
@@ -116,12 +115,9 @@ def train_mpnn(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence
                 direcs=direcs, 
                 dft='siesta',
                 spec=spec,
-                optword=optword,
-                opt=opt_,
+                optword=optword,opt=opt_,
                 nn=nn,vdwnn=vdwnn,
-                bo_layer=bo_layer,
-                mf_layer=mf_layer,
-                be_layer=be_layer,
+                bo_layer=bo_layer,mf_layer=mf_layer,be_layer=be_layer,
                 EnergyFunction=EnergyFunction,
                 MessageFunction=MessageFunction,
                 messages=messages,
@@ -129,7 +125,8 @@ def train_mpnn(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence
                 pkl=False,
                 batch_size=batch,
                 losFunc='n2',
-                bo_penalty=bo_penalty,
+                penalty=penalty,
+                regularize=regularize,lambda_pen=lambda_pen,
                 nanv=nanv,
                 bom=bom,bore=bore,spv_bm=spv_bm,spv_be=spv_be,
                 weight=weight,mefac=mefac,
@@ -137,9 +134,9 @@ def train_mpnn(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence
                 lossConvergence=lossConvergence) # Loss Functon can be n2,abs,mse,huber
 
     loss,accu,accMax,i,zpe =rn.run(learning_rate=1.0e-4,
-                               step=step,
-                               print_step=10,
-                               writelib=writelib) 
+                                   step=step,
+                                   print_step=10,
+                                   writelib=writelib) 
 
     libstep = int(i - i%writelib)
 
